#!/bin/bash
# this script relies on a iostat-output in the form:
# Device:         rrqm/s   wrqm/s     r/s     w/s    rkB/s    wkB/s avgrq-sz avgqu-sz   await  svctm  %util
# testet with iostat delivered with package sysstat 9.0.x

PROC=/usr/bin/iostat
INTERVAL=1

for arg in $@
do
    IFS='=' read -ra ARGS <<< "$arg"
    if   [ "${ARGS[0]}" == "-b" -o "${ARGS[0]}" == "--binary" ]; then PROC=${ARGS[1]}
    elif [ "${ARGS[0]}" == "-i" -o "${ARGS[0]}" == "--interval" ]; then INTERVAL=${ARGS[1]}
    else
        echo ERROR: Argument \'${ARGS[0]}\' not recognized!
        exit 1
    fi
done

resultmsg=""
xmlresult=`cat <<EOF
<?xml version="1.0" encoding='UTF-8'?>
<prtg>
EOF
`

if [ -f $PROC ]; then
  result=$($PROC -xd $INTERVAL 2 | tail -n +2 | grep -v Device:)
  result=${result//,/.}
  while read line; do
    if [ -n "$line" ]; then
      desc=$(echo $line | awk '{print $1}')
      if [[ ! $avg =~ $desc ]]; then
        name_ext=" (since startup)"
      else
        name_ext=""
      fi
      avg+="$desc "

      xmlresult+=`cat <<EOF

  <result>
    <channel>Read Requests per second ${desc}${name_ext}</channel>
    <float>1</float>
    <unit>count</unit>
    <value>$(echo $line | awk '{print $2}')</value>
  </result>
  <result>
    <channel>Write Requests per second ${desc}${name_ext}</channel>
    <float>1</float>
    <unit>count</unit>
    <value>$(echo $line | awk '{print $3}')</value>
  </result>
  <result>
    <channel>Reads per second ${desc}${name_ext}</channel>
    <float>1</float>
    <unit>count</unit>
    <value>$(echo $line | awk '{print $4}')</value>
  </result>
  <result>
    <channel>Writes per second ${desc}${name_ext}</channel>
    <float>1</float>
    <unit>count</unit>
    <value>$(echo $line | awk '{print $5}')</value>
  </result>
  <result>
    <channel>Read kB per second ${desc}${name_ext}</channel>
    <float>1</float>
    <unit>speeddisk</unit>
    <speedsize>kilo</speedsize>
    <value>$(echo $line | awk '{print $6}')</value>
  </result>
  <result>
    <channel>Write KB per second ${desc}${name_ext}</channel>
    <float>1</float>
    <unit>speeddisk</unit>
    <speedsize>kilo</speedsize>
    <value>$(echo $line | awk '{print $7}')</value>
  </result>
  <result>
    <channel>Average Request Size in Sectors ${desc}${name_ext}</channel>
    <float>1</float>
    <unit>count</unit>
    <value>$(echo $line | awk '{print $8}')</value>
  </result>
  <result>
    <channel>Average Queue Length ${desc}${name_ext}</channel>
    <float>1</float>
    <unit>count</unit>
    <value>$(echo $line | awk '{print $9}')</value>
  </result>
  <result>
    <channel>Average Wait Time ${desc}${name_ext}</channel>
    <float>1</float>
    <unit>timeseconds</unit>
    <value>$(echo $line | awk '{printf("%.3f", $10/1000)}')</value>
  </result>
  <result>
    <channel>Average ServiceTime ${desc}${name_ext}</channel>
    <float>1</float>
    <unit>timeseconds</unit>
    <value>$(echo $line | awk '{printf("%.3f", $11/1000)}')</value>
  </result>
  <result>
    <channel>CPU Utilisation for IO ${desc}${name_ext}</channel>
    <float>1</float>
    <unit>Percent</unit>
    <value>$(echo $line | awk '{print $12}')</value>
  </result>
EOF
`
    fi
  done <<< "$result"
  if [ -z "$resultmsg" ]; then
    resultmsg="OK"
  fi

  xmlresult+=`cat <<EOF

  <text>$resultmsg</text>
</prtg>
EOF
`

else
  xmlresult+=`cat <<EOF

  <error>1</error>
  <text>This sensor is not supported by your system, missing $PROC</text>
</prtg>
EOF
`
fi

echo "$xmlresult"

exit 0
